-- ************************************************************************
-- interface for binary file operations (PHP style)
--
-- Alternative to movie script FILE_BINARYIO
-- uses fileIO xtra instead of binaryIO (free, but SLOW)
-- only suitable for small files!
--
-- last changes: 25.02.2005
--**************************************
property fp

on new me
	fp = xtra("fileIO").new()
	return me
end

--------------------------------------
-- 
--------------------------------------
on fopen (me, nil, tFile, mode)
  case (mode) of
    "r","rb": fp.openFile(tFile, 1)
    "w","wb":
      --if not fileExists(tFile) then 
      fp.createFile(tFile)
      fp.openFile(tFile, 2)
    "rw","wr","rwb","wrb": fp.openFile(tFile, 0)
    "a","ab":
      fp.openFile(tFile, 2)
      fp.setPosition(fp.getLength())
    otherwise:
      return 0
  end case
  
  if (fp.status()<>0) then fp = 0
  else return 1 --return fp
end

--------------------------------------
-- 
--------------------------------------
on fclose (me)
  fp.closeFile()
end

--------------------------------------

--------------------------------------
on fread(me, nil, n)
  if voidP(n) then n = fp.getLength() - fp.getPosition()
  data = ""
  repeat with i = 1 to n
    c = fp.readChar()
    if fp.status()<>0 then exit repeat
    put numtochar(chartonum(c)) after data
  end repeat
  return data
end

--------------------------------------
-- 
--------------------------------------
on fwrite (me, nil, str)
  fp.writeString(str)
end

--------------------------------------
-- 
--------------------------------------
on ftell (me)
  return fp.getPosition()
end

--------------------------------------
-- 
--------------------------------------
on fseek (me, nil, pos, whence)
  if voidP(whence) then whence = #SEEK_SET
  case (whence) of
    #SEEK_SET: -- Setzt Position gleich offset bytes.
      fp.setPosition(pos)  
    #SEEK_CUR: -- Setzt Position auf die aktuelle Stelle plus offset.
      fp.setPosition(fp.getPosition() + pos) 
    #SEEK_END: -- Setzt die Position ans Ende der Datei plus offset.
      fp.setPosition(fp.getLength() + pos) 
  end case
  
  return (fp.status()=0)
end

--------------------------------------
-- 
--------------------------------------
on fsize (me)
  return fp.getLength()
end

--------------------------------------
-- truncates file to specified size
--------------------------------------
on ftruncate (me, nil, aSize)
  data = ""
  repeat with i = 1 to aSize
    c = fp.readChar()
    if fp.status()<>0 then exit repeat
    put chr(ord(c)) after data
  end repeat
  
  fn = fp.fileName()
  fp.delete()
  fp.createFile(fn)
  fp.openFile(fn, 2)
  fp.writeString(data)
  
  return (fp.status()=0)
end

--------------------------------------
-- 
--------------------------------------
on fdelete (me, nil, tFile)
  fp.openFile(tFile, 0)
  err = fp.status()
  if not (err) then fp.delete()
  fp = 0
end

--------------------------------------
-- 
--------------------------------------
on fileExists (me, nil, tFile)
  fp.openFile(tFile, 1)
  stat = fp.status()
  fp.closeFile()
  fp = 0
  if stat=-37 then
    return 0
  else
    return 1
  end if
end

--------------------------------------
-- reads whole file, returns (binary) string
--------------------------------------
on file_get_contents (me, nil, tFile, mode)  
  if voidP(mode) then mode="rb"
  if not objectP(fp) then return -1
  fp.openFile(tFile,1)
  --  if (fio.status()<>0) then
  --    die("Error: Can't open "&tFile)
  --  end if
  if mode="r" then -- pure ascii data, no numtochar(0)
    s = fp.readFile()
  else
    s=""
    repeat while true
      c=fp.readChar()
      if fp.status()<>0 then exit repeat
      put numtochar(chartonum(c)) after s
    end repeat
  end if
  fp.closeFile()
  return s
end

--------------------------------------
-- saves (binary) string as file
--------------------------------------
on file_put_contents (me, nil, tFile, tString, tType, tCreator)
  if not objectP(fp) then return -1
  
  fp.openFile(tFile, 1)
  err = fp.status()
  if not (err) then fp.delete()
  else if (err and not (err = -37)) then return err
  
  fp.createFile(tFile)
  err = fp.status()
  if (err) then return err
  
  fp.openFile(tFile, 2)
  err = fp.status()
  if (err) then return err
  fp.writeString(tString)
  if stringP(tType) and stringP(tCreator) then fp.setFinderInfo(tType & tCreator)
  fp.closeFile()
end